<?php

/* * *************************************************************
 *  Copyright notice
 *
 *  (c) 2012 
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 * ************************************************************* */

require_once t3lib_extMgm::extPath("ext_bibsonomy_csl") . "Classes/Lib/Url.php";
require_once t3lib_extMgm::extPath("ext_bibsonomy_csl") . "Classes/Lib/CurlHttpRequest.php";
require_once t3lib_extMgm::extPath("ext_bibsonomy_csl") . "Classes/Lib/CurlHttpResponse.php";

/**
 *
 *
 * @package ext_bibsonomy_csl
 * @license http://www.gnu.org/licenses/gpl.html GNU General Public License, version 3 or later
 * @author Sebastian Böttger
 * 
 */
class Tx_ExtBibsonomyCsl_Domain_Repository_TagRepository extends Tx_Extbase_Persistence_Repository {

	/**
	 *
	 * @var Tx_ExtBibsonomyCsl_Url 
	 */
	protected $url;

	/**
	 * Fetches publications from Bibsonomy API, depending on the settings of the 
	 * front-end plugin.
	 * 
	 * @param array $settings
	 * @return array<Tx_ExtBibsonomyCsl_Domain_Model_Publication>
	 */
	public function fetchTags($settings) {

		$components = parse_url($settings["bib_tag_server"]);
		$components["user"] = $settings["bib_tag_login_name"];
		$components["pass"] = $settings["bib_tag_api_key"];
		$components["path"] = "/api/tags";
		$qry = array();

		if($settings["bib_tag_content_value"] != "" ) {
			$qry[$settings["bib_tag_content_key"]] = $settings["bib_tag_content_value"];
		}
		else {
			throw new Tx_ExtBibsonomyCsl_Domain_Exception_HttpException("Inaccurate API request");
		}
		
		$qry["end"] = $settings["bib_tag_maxcount"];
		$qry["order"] = "frequency";
		$qry["format"] = "json";
		
		$components["query"] = http_build_query($qry);
			
		$url = Tx_ExtBibsonomyCsl_Url::initInstanceWithComponents($components);
		
		//fetches the JSON string 
		$jsonString = $this->getBibsonomyJsonString($url);
		
		//builds json object
		$tags_summit = json_decode($jsonString);
		
		if ($tags_summit == null) {
			throw new Tx_ExtBibsonomyCsl_Domain_Exception_JsonDecodeException();
		}

		$tags = array();
		$taglist = $tags_summit->tags->tag;
		
		
		foreach ($taglist as $jsonObject) {

			try {

				$tag = $this->objectManager->create('Tx_ExtBibsonomyCsl_Domain_Model_Tag');
			} catch (Tx_Extbase_Object_Exception_WrongScopeException $e) {

				$tag = new Tx_ExtBibsonomyCsl_Domain_Model_Tag();
			}
			$tag->setHref($jsonObject->href);
			$tag->setName($jsonObject->name);
			
			$tag->setUrl($settings["bib_tag_server"].$settings["bib_tag_content_key"]."/".$settings["bib_tag_content_value"]."/".$jsonObject->name);
			$tag->setUsercount($jsonObject->usercount);
			$tag->setGlobalcount($jsonObject->globalcount);
			$tags[] = $tag;
		}
		return $tags;
	}

	/**
	 * Fetches the JSON string after running the request with the given URL.
	 * @param string $url
	 * @return String jsonString
	 */
	protected function getBibsonomyJsonString($url) {

		//creates request object
		$request = new Tx_ExtBibsonomyCsl_CurlHttpRequest($url);

		//gets response object
		$response = $request->send();

		if ($response == null) {
			throw new Tx_ExtBibsonomyCsl_Domain_Exception_HttpException(
					"HTTP Error. Maybe this happens because the server connection is blocked by a firewall.\n"
					. "Please contact your webmaster!",
					14
			);
		} 
		//returns body message of response
		return $response->getBody();
	}

	/**
	 * returns url
	 * @return Tx_ExtBibsonomyCsl_Url
	 */
	public function getUrl() {
		return $this->url;
	}

}
